from ..ConexAPI import ConexAPI
from . import DebugParser
import logging


_logger = logging.getLogger(__name__)


def _execute_migrate(args, conex_api: ConexAPI):
    _logger.debug("_execute_migrate")

    conex_api.migrate(args.pattern_or_reference, args.src_remote, args.dst_remote)


def setup_command(subparsers, conex_api: ConexAPI):
    parser = subparsers.add_parser("migrate", help='''
        Copies recipes and binary packages from one remote to another by 
        first downloading and then uploading them. NOTE: The downloaded 
        recipes and binary packages are placed in your local cache and are 
        not removed after upload.''')
    parser.set_defaults(command_func=lambda args: _execute_migrate(args, conex_api))

    parser.add_argument("pattern_or_reference", help='''
        Pattern or package recipe reference, e.g., 'boost/*', 'MyPackage/1.2@user/channel' ''')

    parser.add_argument("src_remote", help='''Download from this remote''')
    parser.add_argument("dst_remote", help='''Upload to this remote''')

    DebugParser.setup(parser)



from ..ConexAPI import ConexAPI
from . import DebugParser


def _execute_upload(args, conex_api: ConexAPI):
    conex_api.upload(args.path, args.lock, args.remote, args.all)


def setup_command(subparsers, conex_api: ConexAPI):
    parser = subparsers.add_parser("upload", help='''
        Uploads recipes and binary packages to a remote''')
    parser.set_defaults(command_func=lambda args: _execute_upload(args, conex_api))

    parser.add_argument("path", help='''
        Path to a folder containing a conanfile.py or conan.lock file (whichever is present). 
        If the folder has both files it will select the conanfile. Use --lock to force the
        selection of the conan.lock file.''')

    parser.add_argument("--lock", default=False, action='store_true', help="Only look for a lock file in the specified path")
    parser.add_argument("-r", "--remote", help="upload to this specific remote")
    parser.add_argument('--no-all', dest='all', default=True, action='store_false',  help="Upload both package recipe and packages")

    DebugParser.setup(parser)



from typing import Optional, Union
from ..types.Reference import Reference
import subprocess


class EditableSubCommand:

    # noinspection PyMethodMayBeStatic
    def add(self, path: str, reference: Union[str, Reference], layout: Optional[str] = None) -> int:
        if isinstance(reference, Reference):
            reference = str(reference)

        command = ["conan", "editable", "add", path, reference]
        if layout:
            command += ["-l", layout]

        result = subprocess.run(command)
        return result.returncode

    # noinspection PyMethodMayBeStatic
    def remove(self, reference: Union[str, Reference]) -> int:
        if isinstance(reference, Reference):
            reference = str(reference)

        command = ["conan", "editable", "remove", reference]
        result = subprocess.run(command)
        return result.returncode

    def list(self) -> str:
        command = ["conan", "editable", "list"]
        result = subprocess.run(command, capture_output=True, text=True, check=True)
        return result.stdout.strip()



from typing import Optional


class Reference:
    def __init__(self, name: str, version: str, user: Optional[str], channel: Optional[str]):
        self._name = name
        self._version = version
        self._user = user
        self._channel = channel

        if (self._user and not self._channel) or (not self._user and self._channel):
            raise ValueError()

    def __str__(self):
        if self._user:
            return f"{self._name}/{self._version}@{self._user}/{self._channel}"
        else:
            return f"{self._name}/{self._version}@"



from typing import List, Type, TypeVar
import json


T = TypeVar('T', bound='LockFile')


class LockFile:
    @classmethod
    def load(cls: Type[T], file_path: str) -> T:
        with open(file_path, "r") as file:
            json_lock = json.load(file)

        LockFile._check_version(json_lock)

        lock_file = LockFile()
        lock_file._packages = LockFile._get_packages(json_lock)
        return lock_file

    def __init__(self):
        self._packages = []

    @property
    def packages(self) -> List[str]:
        return self._packages

    @staticmethod
    def _check_version(json_lock):
        version = json_lock["version"]
        if version != "0.4":
            raise ImportError(
                "ERROR: conan.lock version {} is not supported".format(version)
            )

    @staticmethod
    def _get_packages(json_lock):
        json_nodes = json_lock["graph_lock"]["nodes"]

        requires = set()
        package_dict = {}

        i = 0
        json_node = LockFile._get_node_by_index(json_nodes, i)
        while json_node is not None:

            # first node is root node
            if i != 0 and "ref" in json_node:
                ref = json_node["ref"]
                prev = json_node["prev"]
                package_id = json_node["package_id"]

                package_reference = "{}:{}#{}".format(ref, package_id, prev)
                package_dict[str(i)] = package_reference

            if "requires" in json_node:
                requires |= set(json_node["requires"])

            i += 1
            json_node = LockFile._get_node_by_index(json_nodes, i)

        packages = []
        for i in requires:
            packages.append(package_dict[i])

        return packages

    @staticmethod
    def _get_node_by_index(json_nodes, index):
        try:
            return json_nodes[str(index)]
        except KeyError:
            return None



import subprocess
import tempfile
import json
import os


class ConfigSubCommand:
    def __init__(self):
        pass

    # noinspection PyMethodMayBeStatic
    def home(self) -> str:
        temp_json = None

        try:
            temp_json = tempfile.NamedTemporaryFile(suffix=".json", delete=False)
            temp_json.close()

            command = ["conan", "config", "home", "--json", temp_json.name]

            subprocess.run(command, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)

            with open(temp_json.name) as jsonfile:
                json_values = json.load(jsonfile)

            return json_values["home"]

        finally:
            if temp_json and os.path.exists(temp_json.name):
                os.unlink(temp_json.name)



from ..types.Reference import Reference
from typing import List, Union, Dict
import subprocess
import tempfile
import json
import os


class InspectCommand:
    def inspect(self, path_or_reference: Union[str, Reference], attributes: Union[str, List[str]] = None) -> Dict[str, str]:
        temp_json = None

        try:
            command = ["conan", "inspect"]
            if attributes:
                command += self._attributes(attributes)

            temp_json = tempfile.NamedTemporaryFile(suffix=".json", delete=False)
            temp_json.close()

            command += ["--json", temp_json.name]

            if isinstance(path_or_reference, Reference):
                command.append(str(path_or_reference))
            elif isinstance(path_or_reference, str):
                command.append(path_or_reference)
            else:
                raise ValueError()

            subprocess.run(command, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)

            with open(temp_json.name) as jsonfile:
                return json.load(jsonfile)

        finally:
            if temp_json and os.path.exists(temp_json.name):
                os.unlink(temp_json.name)

    def _attributes(self, values: Union[str, List[str]]):
        params = []

        if isinstance(values, str):
            params += ["--attribute", values]
        elif isinstance(values, list):
            for value in values:
                params += self._attributes(value)
        else:
            print(type(values))
            raise ValueError()

        return params



from typing import Optional, Union, List, Dict
from conanapi import Reference, LockFile


class _FakeUtilities:
    def __init__(self):
        self.lockfile_packages = []

    def load_lockfile(self, file_path: str):
        lock_file = LockFile()
        lock_file._packages = self.lockfile_packages
        return lock_file


class _FakeConfigCommand:
    def __init__(self):
        self._home = None

    def set_home(self, file_path):
        self._home = file_path

    def home(self) -> str:
        return self._home


class _FakeEditableCommand:
    def __init__(self):
        self._editables = {}

    @property
    def editables(self):
        return self._editables

    def is_editable(self, reference: Union[str, Reference]):
        return reference in self._editables

    def add(self, path: str, reference: Union[str, Reference], layout: Optional[str] = None):
        item = {
            "path": path,
            "reference": str(reference),
            "layout": layout
        }
        self._editables[item["reference"]] = item

    def remove(self, reference: Union[str, Reference]):
        self._editables.pop(str(reference))

    def list(self):
        output = ""
        for item in self._editables.values():
            output += f"{item['reference']}\n"
            output += f"    Path: {item['path']}\n"
            output += f"    Layout: {item['layout']}\n"
        return output


class _FakeInspectCommand:
    def __init__(self):
        self.attributes = {}

    def inspect(self, path_or_reference: Union[str, Reference], attributes: Union[str, List[str]] = None) -> Dict[str, str]:
        return self.attributes


class _FakeUploadCommand:
    def __init__(self):
        self.uploads = []

    def upload(self, pattern_or_reference: str, remote: str = None, upload_all: bool = False):
        item = {
            "pattern_or_reference": pattern_or_reference,
            "remote": remote,
            "upload_all": upload_all
        }

        self.uploads.append(item)


class FakeConanAPI:
    def __init__(self):
        self._utilities = _FakeUtilities()
        self._config = _FakeConfigCommand()
        self._editable = _FakeEditableCommand()
        self._inspect = _FakeInspectCommand()
        self._upload = _FakeUploadCommand()

    @property
    def fake_utilities(self):
        return self._utilities

    @property
    def fake_config(self):
        return self._config

    @property
    def fake_editable(self):
        return self._editable

    @property
    def fake_inspect(self):
        return self._inspect

    @property
    def fake_upload(self):
        return self._upload

    @property
    def utilities(self):
        return self._utilities

    @property
    def config(self):
        return self._config

    @property
    def editable(self):
        return self._editable

    @property
    def inspect(self):
        return self._inspect.inspect

    @property
    def upload(self):
        return self._upload.upload



import subprocess


class UploadCommand:
    # noinspection PyMethodMayBeStatic
    def upload(self, pattern_or_reference: str, remote: str = None, upload_all: bool = False, suppress_output: bool = False):
        command = ["conan", "upload", pattern_or_reference]
        if remote is not None:
            command += ["--remote", remote]

        if upload_all:
            command += ["--all"]

        if suppress_output:
            subprocess.run(command, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
        else:
            subprocess.run(command)



from .commands.ConfigSubCommand import ConfigSubCommand
from .commands.EditableSubCommand import EditableSubCommand
from .commands.InspectCommand import InspectCommand
from .commands.UploadCommand import UploadCommand
from .commands.DownloadCommand import DownloadCommand
from .commands.SearchCommand import SearchCommand
from .internal.Utilities import Utilities


class ConanAPI:
    def __init__(self):
        self._utilities = Utilities()
        self._config = ConfigSubCommand()
        self._editable = EditableSubCommand()
        self._inspect = InspectCommand()
        self._upload = UploadCommand()
        self._download = DownloadCommand()
        self._search = SearchCommand()

    @property
    def utilities(self):
        return self._utilities

    @property
    def config(self):
        return self._config

    @property
    def editable(self):
        return self._editable

    @property
    def inspect(self):
        return self._inspect.inspect

    @property
    def upload(self):
        return self._upload.upload

    @property
    def download(self):
        return self._download.download

    @property
    def search(self):
        return self._search.search



from ..commands.ConfigSubCommand import ConfigSubCommand
from conanapi import Reference


class Utilities:
    def __init__(self, conan, config: ConfigSubCommand):
        self._conan_inspect = conan.inspect
        self._conan_utilities = conan.utilities
        self._config = config

    def create_reference(self, path_to_conanfile):
        package_info = self._conan_inspect(path_to_conanfile, attributes=["name", "version"])

        name = package_info["name"]
        version = package_info["version"]
        user = self._config.get("user")
        channel = self._config.get("channel")

        return Reference(name, version, user, channel)

    def load_lockfile(self, file_path: str):
        return self._conan_utilities.load_lockfile(file_path)



from .ConfigSubCommand import ConfigSubCommand
from ..internal.Utilities import Utilities
import re


class EditableSubCommand:
    def __init__(self, conan, utils: Utilities, config: ConfigSubCommand):
        self._conan_editable = conan.editable
        self._utils = utils
        self._config = config

    def add(self, path_to_conanfile, path_to_layout=None):
        reference = self._utils.create_reference(path_to_conanfile)
        self._conan_editable.add(path_to_conanfile, reference, path_to_layout)

    def remove(self, path_to_conanfile, remove_all):
        if remove_all is not None:
            self._remove_all()
        elif path_to_conanfile is not None:
            self._remove_package(path_to_conanfile)
        else:
            raise ValueError('Unsupported operation')

    def _remove_package(self, path_to_conanfile):
        reference = self._utils.create_reference(path_to_conanfile)
        self._conan_editable.remove(reference)

    def _remove_all(self):
        output = self._conan_editable.list()
        if not output:
            print("No editable packages found. Nothing to do.")
            return

        # Use regex to find the package names (the first line of each 3-line block)
        # Based on the sample, items look like 'name/version@user/channel'
        # We look for lines that don't start with 'Path:' or 'Layout:'
        references = re.findall(r'^(\S+/\S+@\S+/\S+)', output, re.MULTILINE)
        if not references:
            print("No editable packages identified in the output.")
            return

        print(f"Found {len(references)} editable packages:")
        for reference in references:
            print(f"  - {reference}")

        print()

        for reference in references:
            self._conan_editable.remove(reference)



import argparse
import sys


class ArgumentParser(argparse.ArgumentParser):
    def error(self, message):
        sys.stderr.write('ERROR: %s\n\n' % message)
        self.print_help()
        sys.exit(2)



from ..internal import Utilities
from enum import Enum
import logging


class DownloadCommand:
    class InputType(Enum):
        PATTERN = 1,
        REFERENCE = 2

    def __init__(self, conan, conex_utils: Utilities):
        self._logger = logging.getLogger(__name__)

        self._conan = conan
        self._utils = conex_utils

    def download(self, pattern_or_reference: str, remote: str = None):
        self._download(pattern_or_reference, remote)

    def _download(self, pattern_or_reference: str, remote: str = None):
        input_type = self._get_input_type(pattern_or_reference)

        func = {
            DownloadCommand.InputType.PATTERN: self._download_from_pattern,
            DownloadCommand.InputType.REFERENCE: self._download_from_reference
        }

        # noinspection PyArgumentList
        return func[input_type](pattern_or_reference, remote)

    def _download_from_pattern(self, pattern: str, remote: str):
        pattern_results = self._conan.search(pattern, remote)
        for reference in pattern_results:
            self._download_from_reference(reference, remote)

        return pattern_results

    def _download_from_reference(self, reference: str, remote: str):
        self._conan.download(reference, remote)
        return [reference]

    @staticmethod
    def _get_input_type(pattern_or_reference: str):
        if "*" in pattern_or_reference:
            return DownloadCommand.InputType.PATTERN
        else:
            return DownloadCommand.InputType.REFERENCE



from pprint import pformat
import subprocess
import tempfile
import json
import os
import logging


class SearchCommand:
    def __init__(self):
        self._logger = logging.getLogger(__name__)

    # noinspection PyMethodMayBeStatic
    def search(self, pattern_or_reference: str, remote: str = None):
        temp_json = None

        try:
            command = ["conan", "search"]
            if remote is not None:
                command += ["--remote", remote]

            temp_json = tempfile.NamedTemporaryFile(suffix=".json", delete=False)
            temp_json.close()

            command += ["--json", temp_json.name]
            command.append(pattern_or_reference)

            subprocess.run(command, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)

            with open(temp_json.name) as jsonfile:
                json_results = json.load(jsonfile)
                self._logger.debug(f"json output:\n{json.dumps(json_results, indent=4)}")

            error = json_results["error"]
            if error:
                self._logger.error("An unknown error occurred with conan search")
                raise Exception("Error occurred in search")

            packages = []
            results = json_results["results"]
            for result in results:
                items = result["items"]
                for item in items:
                    package = item["recipe"]["id"]
                    packages.append(package)

            self._logger.debug(f"packages:\n{pformat(packages)}")
            return packages

        finally:
            if temp_json and os.path.exists(temp_json.name):
                os.unlink(temp_json.name)



import tomlkit
import os


class ConfigSubCommand:
    def __init__(self, conan, config_filename):
        self._conan_config = conan.config
        self._config_filename = config_filename

    def _initialize_config(self):
        conex_config = {
            "user": None,
            "channel": None
        }

        self._save_config(conex_config)

    def _save_config(self, conex_config):
        for pair in conex_config.items():
            item = pair[0]
            value = pair[1]
            if value is None:
                conex_config[item] = ""

        conan_config_home = self._conan_config.home()
        conex_config_path = os.path.join(conan_config_home, self._config_filename)

        with open(conex_config_path, "w") as conex_config_file:
            tomlkit.dump(conex_config, conex_config_file)

    def _load_config(self):
        conan_config_home = self._conan_config.home()
        conex_config_path = os.path.join(conan_config_home, self._config_filename)

        if not os.path.exists(conex_config_path):
            self._initialize_config()

        with open(conex_config_path, "r") as conex_config_file:
            toml_doc = tomlkit.load(conex_config_file)

        return ConfigSubCommand._tomlkit_to_popo(toml_doc)

    @staticmethod
    def _tomlkit_to_popo(d):
        try:
            result = getattr(d, "value")
        except AttributeError:
            result = d

        if isinstance(result, list):
            result = [ConfigSubCommand._tomlkit_to_popo(x) for x in result]
        elif isinstance(result, dict):
            result = {
                ConfigSubCommand._tomlkit_to_popo(key): ConfigSubCommand._tomlkit_to_popo(val) for key, val in result.items()
            }
        elif isinstance(result, tomlkit.items.Integer):
            result = int(result)
        elif isinstance(result, tomlkit.items.Float):
            result = float(result)
        elif isinstance(result, tomlkit.items.String):
            result = str(result)
            if result == "":
                result = None
        elif isinstance(result, tomlkit.items.Bool):
            result = bool(result)

        return result

    def load(self):
        return self._load_config()

    def get(self, item):
        _conex_config = self._load_config()
        if item not in _conex_config:
            raise ValueError()

        return _conex_config[item]

    def set(self, item: str, value):
        _conex_config = self._load_config()
        _conex_config[item] = value
        self._save_config(_conex_config)

    def remove(self, item):
        _conex_config = self._load_config()
        if item not in _conex_config:
            return ValueError()

        _conex_config.pop(item)
        self._save_config(_conex_config)



from .DownloadCommand import DownloadCommand
from .UploadCommand import UploadCommand
import logging


class MigrateCommand:
    def __init__(self, download: DownloadCommand, upload: UploadCommand):
        self._logger = logging.getLogger(__name__)

        self._download = download
        self._upload = upload

    # noinspection PyProtectedMember
    def migrate(self, pattern_or_reference: str, source: str, destination: str):
        download_packages = self._download._download(pattern_or_reference, source)
        self._upload._upload(download_packages, destination, True)



import subprocess


class DownloadCommand:
    # noinspection PyMethodMayBeStatic
    def download(self, reference: str, remote: str = None, suppress_output: bool = False):

        command = ["conan", "download", reference]
        if remote is not None:
            command += ["--remote", remote]

        if suppress_output:
            subprocess.run(command, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
        else:
            subprocess.run(command)



from typing import List
from ..internal import Utilities
import os


class UploadCommand:
    def __init__(self, conan, conex_utils: Utilities):
        self._conan_upload = conan.upload
        self._utils = conex_utils

    def upload(self, path_to_conanfile_or_lock: str, lockfile_only: bool = False, remote: str = None, upload_all: bool = True):
        conanfile_or_lockfile, is_lock_file = self._lookup_file(path_to_conanfile_or_lock, lockfile_only)

        if is_lock_file:
            lock_file = self._utils.load_lockfile(conanfile_or_lockfile)
            self._upload(lock_file.packages)

        else:
            reference = self._utils.create_reference(path_to_conanfile_or_lock)
            self._upload([str(reference)], remote, upload_all)

    def _upload(self, references: List[str], remote: str = None, upload_all: bool = True):
        for reference in references:
            self._conan_upload(reference, remote, upload_all)

    def _lookup_file(self, path_to_conanfile_or_lock: str, lockfile_only: bool):
        absolute_path = os.path.abspath(path_to_conanfile_or_lock)

        conanfile_path = None

        if not lockfile_only:
            conanfile_path = self._lookup_conanfile(absolute_path)

        if conanfile_path is not None:
            return conanfile_path, False

        lockfile_path = self._lookup_lockfile(absolute_path)
        if lockfile_path is None:
            raise ValueError()

        return lockfile_path, True

    # noinspection PyMethodMayBeStatic
    def _lookup_conanfile(self, absolute_path):
        conanfile_path = os.path.join(absolute_path, "conanfile.py")
        if os.path.exists(conanfile_path):
            return conanfile_path
        else:
            return None

    # noinspection PyMethodMayBeStatic
    def _lookup_lockfile(self, absolute_path):
        conanfile_path = os.path.join(absolute_path, "conan.lock")
        if os.path.exists(conanfile_path):
            return conanfile_path
        else:
            return None



from ..ConexAPI import ConexAPI
from . import DebugParser


def _print_config_items_and_values(conex_api: ConexAPI):
    _convex_config = conex_api.config.load()

    print(f"conex configuration:")
    for pair in _convex_config.items():
        print(f"    {pair[0]} = {pair[1]}")


def _setup_list_command(subparsers, conex_api: ConexAPI):
    parser = subparsers.add_parser("show", help="Show all the specified configuration items with their values")
    parser.set_defaults(command_func=lambda args: _print_config_items_and_values(conex_api))


def _setup_get_command(subparsers, conex_api: ConexAPI):
    parser = subparsers.add_parser("get", help="Get the value of a configuration item")

    parser.add_argument("item", help="item to print")
    parser.set_defaults(command_func=lambda args: conex_api.config.get(args.item))


def _set_config(input_: str, conex_api: ConexAPI):
    tokens = input_.split("=")
    item = tokens[0]
    value = tokens[1]

    if input_ != f"{item}={value}":
        raise ValueError()

    if value == "None":
        value = None

    conex_api.config.set(item, value)


def _setup_set_command(subparsers, conex_api: ConexAPI):
    parser = subparsers.add_parser("set", help="Set a value for a configuration item")

    parser.add_argument("item", help="'item=value' to set")
    parser.set_defaults(command_func=lambda args: _set_config(args.item, conex_api))


def _setup_remove_command(subparsers, conex_api: ConexAPI):
    parser = subparsers.add_parser("remove", help="Remove an existing config element")

    parser.add_argument("item", help="item to remove")
    parser.set_defaults(command_func=lambda args: conex_api.config.remove(args.item))


def setup_command(subparsers, conex_api: ConexAPI):
    parser = subparsers.add_parser("config", help='''Manages conex configuration.''')

    subparsers = parser.add_subparsers(title="subcommand", dest="subcommand")
    subparsers.required = True

    _setup_list_command(subparsers, conex_api)
    _setup_get_command(subparsers, conex_api)
    _setup_set_command(subparsers, conex_api)
    _setup_remove_command(subparsers, conex_api)

    DebugParser.setup(parser)



from ..ConexAPI import ConexAPI
from . import DebugParser
import logging


_logger = logging.getLogger(__name__)


def _execute_download(args, conex_api: ConexAPI):
    _logger.debug("_execute_download")

    conex_api.download(args.pattern_or_reference, args.remote)


def setup_command(subparsers, conex_api: ConexAPI):
    parser = subparsers.add_parser("download", help='''
        Downloads recipes and binary packages to the local cache, without using settings''')
    parser.set_defaults(command_func=lambda args: _execute_download(args, conex_api))

    parser.add_argument("pattern_or_reference", help='''
        Pattern or package recipe reference, e.g., 'boost/*', 'MyPackage/1.2@user/channel' ''')

    parser.add_argument("-r", "--remote", help="download from this specific remote")

    DebugParser.setup(parser)



from ..ConexAPI import ConexAPI
from . import DebugParser


def _setup_remove_command(subparsers, conex_api: ConexAPI):
    parser = subparsers.add_parser("remove", help="Disable editable mode for a package")
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument("path", nargs="?", help="Remove the package located at the path in the user workspace")
    group.add_argument("--all", action="store_true", help="Remove all editable packages")
    parser.set_defaults(command_func=lambda args: conex_api.editable.remove(args.path, args.all))


def _setup_add_command(subparsers, conex_api: ConexAPI):
    parser = subparsers.add_parser("add", help="Put a package in editable mode")
    parser.add_argument("path", help="Path to the package folder in the user workspace")
    parser.add_argument("-l", "--layout", default=None, help="Relative or absolute path to a file containing the layout")
    parser.set_defaults(command_func=lambda args: conex_api.editable.add(args.path, args.layout))


def setup_command(subparsers, conex_api: ConexAPI):
    parser = subparsers.add_parser("editable", help='''
        Manages editable packages (packages that reside in the user workspace, 
        but are consumed as if they were in the cache).''')

    subparsers = parser.add_subparsers(title="subcommand", dest="subcommand")
    subparsers.required = True

    _setup_add_command(subparsers, conex_api)
    _setup_remove_command(subparsers, conex_api)

    DebugParser.setup(parser)



from .commands.ConfigSubCommand import ConfigSubCommand
from .commands.EditableSubCommand import EditableSubCommand
from .commands.UploadCommand import UploadCommand
from .commands.DownloadCommand import DownloadCommand
from .commands.MigrateCommand import MigrateCommand
from .internal.Utilities import Utilities
from conanapi import ConanAPI


class ConexAPI:
    config_filename = "conex.toml"

    def __init__(self, conan=ConanAPI()):
        self._config = ConfigSubCommand(conan, ConexAPI.config_filename)
        self._utils = Utilities(conan, self._config)
        self._editable = EditableSubCommand(conan, self._utils, self._config)
        self._upload = UploadCommand(conan, self._utils)
        self._download = DownloadCommand(conan, self._utils)
        self._migrate = MigrateCommand(self._download, self._upload)

    @property
    def config(self):
        return self._config

    @property
    def editable(self):
        return self._editable

    @property
    def upload(self):
        return self._upload.upload

    @property
    def download(self):
        return self._download.download

    @property
    def migrate(self):
        return self._migrate.migrate



from conex.cli_parsers import ArgumentParser
from conex.cli_parsers import EditableCommandParser
from conex.cli_parsers import ConfigCommandParser
from conex.cli_parsers import UploadCommandParser
from conex.cli_parsers import DownloadCommandParser
from conex.cli_parsers import MigrateCommandParser
from conex.ConexAPI import ConexAPI
import logging
import sys


__version = "0.3.0"


def _setup_logging():
    logging.basicConfig(level=logging.DEBUG)


def _setup_parser(conex_api: ConexAPI):
    parser = ArgumentParser(prog=__package__)
    parser.add_argument("-v", "--version", action="version", version=__version)

    subparsers = parser.add_subparsers(title="subcommand", dest="subcommand")
    subparsers.required = True

    ConfigCommandParser.setup_command(subparsers, conex_api)
    EditableCommandParser.setup_command(subparsers, conex_api)
    UploadCommandParser.setup_command(subparsers, conex_api)
    DownloadCommandParser.setup_command(subparsers, conex_api)
    MigrateCommandParser.setup_command(subparsers, conex_api)

    return parser


def main(args=None):
    conex_api = ConexAPI()
    parser = _setup_parser(conex_api)

    if not args:
        args = sys.argv[1:]

    parsed = parser.parse_args(args)
    if parsed.debug:
        _setup_logging()

    parsed.command_func(parsed)


if __name__ == "__main__":
    main()
